<!DOCTYPE html>
<html>
  <head>
    <title>Terracotta API Documentation</title>
    <meta charset="utf-8"/>
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <link rel="shortcut icon" href="{{ url_for('static', filename='favicon.ico') }}">
    <style> body { margin: 0; padding: 0; } </style>
  </head>
  <body>
    <redoc spec-url="{{ url_for('.specification') }}" hide-loading></redoc>
    <script src="{{ url_for('static', filename='redoc.min.js') }}"> </script>
  </body>
</html>
